@@ -1,20 +1,25 @@
 # -*- coding: utf-8 -*-
-from jinja2 import TemplateAssertionError
-
+from nose import SkipTest
+import shutil, os
+import json
 import tg
+#import tg.configuration
+
+#tg.configuration.reqlocal_config.push_process_config({})
+
 from tests.test_stack import TestConfig, app_from_config
 from tg.util import Bunch
-from webtest import TestApp
-from pylons import tmpl_context
+from tg._compat import PY3
+from tg.render import RenderGenshi
 
-def setup_noDB(genshi_doctype=None, genshi_method=None, genshi_encoding=None):
+def setup_noDB(genshi_doctype=None, genshi_method=None, genshi_encoding=None, extra={}):
     base_config = TestConfig(folder='rendering', values={
         'use_sqlalchemy': False,
-       'pylons.helpers': Bunch(),
        'use_legacy_renderer': False,
        # this is specific to mako  to make sure inheritance works
        'use_dotted_templatenames': False,
-       'pylons.tmpl_context_attach_args': False
+       'use_toscawidgets': False,
+       'use_toscawidgets2': False
     })
 
     deployment_config = {}
@@ -29,6 +34,7 @@ def setup_noDB(genshi_doctype=None, genshi_method=None, genshi_encoding=None):
     if genshi_encoding:
         deployment_config['templating.genshi.encoding'] = genshi_encoding
 
+    deployment_config.update(extra)
     return app_from_config(base_config, deployment_config)
 
 def test_default_genshi_renderer():
@@ -250,12 +256,16 @@ def test_genshi_sub_inheritance_from_bottom():
     assert "Master template" in resp
 
 def test_chameleon_genshi_base():
+    if PY3: raise SkipTest()
+
     app = setup_noDB()
     resp = app.get('/chameleon_genshi_index')
     assert ("<p>TurboGears 2 is rapid web application development toolkit"
         " designed to make your life easier.</p>") in resp
 
 def test_chameleon_genshi_inheritance():
+    if PY3: raise SkipTest()
+
     try:
         import lxml
     except ImportError:
@@ -265,11 +275,11 @@ def test_chameleon_genshi_inheritance():
     app = setup_noDB()
     try:
         resp = app.get('/chameleon_genshi_inherits')
-    except NameError, e:
+    except NameError as e:
         # known issue with chameleon.genshi 1.0
         if 'match_templates' not in str(e):
             raise
-    except AttributeError, e:
+    except AttributeError as e:
         # known issue with chameleon.genshi 1.3
         if 'XPathResult' not in str(e):
             raise
@@ -279,18 +289,7 @@ def test_chameleon_genshi_inheritance():
 
 def test_jinja_autoload():
     app = setup_noDB()
-    try:
-        resp = app.get('/jinja_autoload')
-
-        # Normally the template should not load, if it does
-        # check if the filter namespace has been polluted.
-        assert not ("POLLUTED" in resp), resp
-    except TemplateAssertionError:
-        # If autoloading is working ok a template
-        # getting a function not in __all__ should raise
-        # a template exception, thus this is the normal behaviour.
-        pass
-
+    resp = app.get('/jinja_autoload', status=500)
 
 def _test_jinja_inherits():
     app = setup_noDB()
@@ -300,15 +299,15 @@ def _test_jinja_inherits():
 def test_jinja_extensions():
     base_config = TestConfig(folder = 'rendering',
                              values = {'use_sqlalchemy': False,
-                                       'pylons.helpers': Bunch(),
                                        'use_legacy_renderer': False,
                                        # this is specific to mako
                                        # to make sure inheritance works
                                        'use_dotted_templatenames': False,
-                                       'pylons.tmpl_context_attach_args': False,
                                        'renderers':['jinja'],
                                        'jinja_extensions': ['jinja2.ext.do', 'jinja2.ext.i18n',
-                                                            'jinja2.ext.with_', 'jinja2.ext.autoescape']
+                                                            'jinja2.ext.with_', 'jinja2.ext.autoescape'],
+                                       'use_toscawidgets': False,
+                                       'use_toscawidgets2': False
                                        }
                              )
     app = app_from_config(base_config)
@@ -328,19 +327,19 @@ def test_jinja_custom_filters():
             from hashlib import md5
         except ImportError:
             from md5 import md5
-        string_hash = md5(value)
+        string_hash = md5(value.encode('ascii'))
         return string_hash.hexdigest()
 
     base_config = TestConfig(folder = 'rendering',
                              values = {'use_sqlalchemy': False,
-                                       'pylons.helpers': Bunch(),
                                        'use_legacy_renderer': False,
                                        # this is specific to mako
                                        # to make sure inheritance works
                                        'use_dotted_templatenames': False,
-                                       'pylons.tmpl_context_attach_args': False,
                                        'renderers':['jinja'],
-                                       'jinja_filters': {'codify': codify}
+                                       'jinja_filters': {'codify': codify},
+                                       'use_toscawidgets': False,
+                                       'use_toscawidgets2': False
                                        }
                              )
     app = app_from_config(base_config)
@@ -357,6 +356,29 @@ def test_mako_renderer():
     resp = app.get('/mako_index')
     assert "<p>This is the mako index page</p>" in resp, resp
 
+def test_mako_renderer_compiled():
+    tg.config['templating.mako.compiled_templates_dir'] = '_tg_tests_mako_compiled/dest'
+    app = setup_noDB()
+    resp = app.get('/mako_index')
+    tg.config.pop('templating.mako.compiled_templates_dir')
+    assert "<p>This is the mako index page</p>" in resp, resp
+
+    assert os.path.exists('_tg_tests_mako_compiled')
+    shutil.rmtree('_tg_tests_mako_compiled', True)
+
+def test_mako_renderer_compiled_existing():
+    os.makedirs('_tg_tests_mako_compiled/dest')
+    test_mako_renderer_compiled()
+
+def test_mako_renderer_compiled_no_access():
+    os.makedirs('_tg_tests_mako_compiled')
+    os.makedirs('_tg_tests_mako_compiled/dest', mode=0o400)
+    test_mako_renderer_compiled()
+
+def test_mako_renderer_compiled_no_access_parent():
+    os.makedirs('_tg_tests_mako_compiled', mode=0o400)
+    test_mako_renderer_compiled()
+
 def test_mako_inheritance():
     app = setup_noDB()
     resp = app.get('/mako_inherits')
@@ -367,13 +389,13 @@ def test_template_override():
 #    app = setup_noDB()
     base_config = TestConfig(folder = 'rendering',
                              values = {'use_sqlalchemy': False,
-                                       'pylons.helpers': Bunch(),
                                        'use_legacy_renderer': False,
                                        # this is specific to mako
                                        # to make sure inheritance works
                                        'use_dotted_templatenames': True,
-                                       'pylons.tmpl_context_attach_args': False,
-                                       'renderers':['genshi']
+                                       'renderers':['genshi'],
+                                       'use_toscawidgets': False,
+                                       'use_toscawidgets2': False
                                        }
                              )
     app = app_from_config(base_config)
@@ -390,17 +412,17 @@ def test_template_override_wts():
 #    app = setup_noDB()
     base_config = TestConfig(folder = 'rendering',
                              values = {'use_sqlalchemy': False,
-                                       'pylons.helpers': Bunch(),
                                        'use_legacy_renderer': False,
                                        # this is specific to mako
                                        # to make sure inheritance works
                                        'use_dotted_templatenames': True,
-                                       'pylons.tmpl_context_attach_args': False,
-                                       'renderers':['genshi']
+                                       'renderers':['genshi'],
+                                       'use_toscawidgets': False,
+                                       'use_toscawidgets2': False
                                        }
                              )
     app = app_from_config(base_config)
-    r = app.get('/template_override_wts', status=302) # ensure with_trailing_slash
+    r = app.get('/template_override_wts', status=301) # ensure with_trailing_slash
     r =app.get('/template_override_wts/')
     assert "Not overridden" in r, r
     r = app.get('/template_override_wts/', params=dict(override=True))
@@ -413,13 +435,13 @@ def test_template_override_wts():
 def test_template_override_content_type():
     base_config = TestConfig(folder = 'rendering',
                              values = {'use_sqlalchemy': False,
-                                       'pylons.helpers': Bunch(),
                                        'use_legacy_renderer': False,
                                        # this is specific to mako
                                        # to make sure inheritance works
                                        'use_dotted_templatenames': True,
-                                       'pylons.tmpl_context_attach_args': False,
-                                       'renderers':['mako', 'genshi']
+                                       'renderers':['mako', 'genshi'],
+                                       'use_toscawidgets': False,
+                                       'use_toscawidgets2': False
                                        }
                              )
     app = app_from_config(base_config)
@@ -458,6 +480,11 @@ def test_template_custom_format_html():
     assert 'html' in resp
     assert resp.content_type == 'text/html'
 
+def test_template_custom_format_nonexisting():
+    app = setup_noDB()
+    resp = app.get('/custom_format?format=csv', status=500)
+    assert 'not a valid custom_format' in resp
+
 def test_template_override_multiple_content_type():
     app = setup_noDB()
     resp = app.get('/template_override_multiple_content_type')
@@ -468,11 +495,19 @@ def test_template_override_multiple_content_type():
         params=dict(override=True))
     assert 'This is the mako index page' in resp
 
+def test_override_template_on_noncontroller():
+    tg.override_template(None, 'this.is.not.a.template')
+
 def test_jinja2_manual_rendering():
     app = setup_noDB()
     tgresp = app.get('/jinja2_manual_rendering')
     pyresp = app.get('/jinja2_manual_rendering?frompylons=1')
-    assert str(tgresp) == str(pyresp)
+    assert str(tgresp) == str(pyresp), str(tgresp) + '\n------\n' + str(pyresp)
+
+def test_no_template():
+    app = setup_noDB()
+    resp = app.get('/no_template_generator')
+    assert '1234' in resp, resp
 
 def test_genshi_manual_render_no_doctype():
     app = setup_noDB()
@@ -499,3 +534,76 @@ def test_genshi_manual_render_html_doctype():
     assert 'content="text/html; charset=utf-8"' in resp
     assert "<hr>" in resp
     assert "<p>Rendered with Genshi.</p>" in resp
+
+def test_genshi_manual_render_svg_doctype():
+    app = setup_noDB()
+    resp = app.get('/genshi_manual_rendering_with_doctype?doctype=svg')
+    assert '<!DOCTYPE svg' in resp
+
+def test_genshi_methods_for_doctype():
+    assert RenderGenshi.method_for_doctype('application/xml') == 'xhtml'
+
+def test_variable_provider():
+    app = setup_noDB(extra={'variable_provider': lambda: {'inject_this_var':5}})
+    resp = app.get('/get_tg_vars')
+    assert 'inject_this_var' in resp
+
+def test_render_hooks():
+    calls = []
+    def render_call_hook(*args, **kw):
+        calls.append(1)
+
+    base_config = TestConfig(folder='rendering', values={
+        'use_sqlalchemy': False,
+        'use_legacy_renderer': False,
+        # this is specific to mako  to make sure inheritance works
+        'use_dotted_templatenames': False,
+        'use_toscawidgets': False,
+        'use_toscawidgets2': False
+    })
+    base_config.register_hook('before_render_call', render_call_hook)
+    base_config.register_hook('after_render_call', render_call_hook)
+    app = app_from_config(base_config)
+    app.get('/')
+
+    assert len(calls) == 2
+
+class TestTemplateCaching(object):
+    def setUp(self):
+        base_config = TestConfig(folder='rendering', values={
+            'use_sqlalchemy': False,
+            'use_legacy_renderer': False,
+            # this is specific to mako  to make sure inheritance works
+            'use_dotted_templatenames': False,
+            'use_toscawidgets': False,
+            'use_toscawidgets2': False,
+            'cache_dir': '.'
+        })
+        self.app = app_from_config(base_config)
+
+    def test_basic(self):
+        resp = self.app.get('/template_caching')
+        current_date = resp.text.split('NOW:')[1].split('\n')[0].strip()
+
+        resp = self.app.get('/template_caching')
+        assert current_date in resp, (current_date, resp.body)
+
+    def test_default_type(self):
+        resp = self.app.get('/template_caching_default_type')
+        current_date = resp.text.split('NOW:')[1].split('\n')[0].strip()
+
+        resp = self.app.get('/template_caching_default_type')
+        assert current_date in resp, (current_date, resp.body)
+
+    def test_template_caching_options(self):
+        resp = self.app.get('/template_caching_options', params={'cache_type':'memory'})
+        resp = json.loads(resp.text)
+        assert resp['cls'] == 'MemoryNamespaceManager', resp
+
+        resp = self.app.get('/template_caching_options', params={'cache_expire':1})
+        resp = json.loads(resp.text)
+        assert resp['cls'] == 'NoImplementation', resp
+
+        resp = self.app.get('/template_caching_options', params={'cache_key':'TEST'})
+        resp = json.loads(resp.text)
+        assert resp['cls'] == 'NoImplementation', resp
