@@ -1,26 +1,97 @@
 # -*- coding: utf-8 -*-
-
 from nose.tools import raises
+from nose import SkipTest
 
-import pylons
-from formencode import validators, Schema
-from simplejson import loads, dumps
+import tg
+import tests
+from json import loads, dumps
 
 from tg.controllers import TGController
 from tg.decorators import expose, validate, before_render
 from tests.base import (TestWSGIController, data_dir,
     make_app, setup_session_dir, teardown_session_dir)
 
-from tw.forms import TableForm, TextField
-from tw.api import WidgetsList
+from tg._compat import PY3, unicode_text, u_
+from tg.validation import TGValidationError
 
-import tw2.core as tw2c
-import tw2.forms as tw2f
+if not PY3:
+    from formencode import validators, Schema
 
-class MovieForm(tw2f.TableForm):
-    title = tw2f.TextField(validator=tw2c.Required)
-    year = tw2f.TextField(size=4, validator=tw2c.IntValidator)
-movie_form = MovieForm(action='save_movie')
+    from tw.forms import TableForm, TextField
+    from tw.api import WidgetsList
+
+    import tw2.core as tw2c
+    import tw2.forms as tw2f
+
+    class MovieForm(tw2f.TableForm):
+        title = tw2f.TextField(validator=tw2c.Required)
+        year = tw2f.TextField(size=4, validator=tw2c.IntValidator)
+    movie_form = MovieForm(action='save_movie')
+
+    class MyForm(TableForm):
+        class fields(WidgetsList):
+            """This WidgetsList is just a container."""
+            title=TextField(validator = validators.NotEmpty())
+            year = TextField(size=4, validator=validators.Int())
+    myform = MyForm("my_form", action='create')
+
+    class Pwd(Schema):
+        pwd1 = validators.String(not_empty=True)
+        pwd2 = validators.String(not_empty=True)
+        chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]
+else:
+    movie_form = None
+    myform = None
+
+    class validators(object):
+        """Simulate Formencode"""
+        Invalid = TGValidationError
+
+        class FancyValidator(object):
+            def _to_python(self, value):
+                return value
+
+            def validate_python(self, value, state=None):
+                if not value:
+                    raise TGValidationError('Empty')
+
+            def to_python(self, value, state=None):
+                try:
+                    pyv = self._to_python(value)
+                except Exception as e:
+                    raise TGValidationError(str(e))
+
+                self.validate_python(pyv, None)
+                return pyv
+
+        class Int(FancyValidator):
+            def _to_python(self, value):
+                try:
+                    return int(value)
+                except:
+                    raise TGValidationError('Must be an integer')
+
+        class Email(FancyValidator):
+            def _to_python(self, value):
+                if '@' not in value:
+                    raise TGValidationError('not email')
+                return value
+
+        class StringBool(FancyValidator):
+            def validate_python(self, value, state=None):
+                return value
+
+            def _to_python(self, value):
+                if not value:
+                    return value
+
+                if value == 'True':
+                    return True
+                raise TGValidationError('Not True')
+
+    class tw2c(object):
+        class IntValidator(validators.Int):
+            pass
 
 def setup():
     setup_session_dir()
@@ -28,23 +99,6 @@ def setup():
 def teardown():
     teardown_session_dir()
 
-class MyForm(TableForm):
-
-    class fields(WidgetsList):
-        """This WidgetsList is just a container."""
-        title=TextField(validator = validators.NotEmpty())
-        year = TextField(size=4, validator=validators.Int())
-
-# then, we create an instance of this form
-myform = MyForm("my_form", action='create')
-
-
-class Pwd(Schema):
-    pwd1 = validators.String(not_empty=True)
-    pwd2 = validators.String(not_empty=True)
-    chained_validators = [validators.FieldsMatch('pwd1', 'pwd2')]
-
-
 class controller_based_validate(validate):
 
     def __init__(self, error_handler=None, *args, **kw):
@@ -61,19 +115,27 @@ class ColonValidator(validators.FancyValidator):
     def validate_python(self, value, state):
         raise validators.Invalid('ERROR: Description', value, state)
 
-class BasicTGController(TGController):
+class ColonLessGenericValidator(object):
+    def validate(self, value, state=None):
+        raise validators.Invalid('Unknown Error', value, {'_the_form':'Unknown Error'})
 
-    @expose('json')
+class BasicTGController(TGController):
+    @expose()
+    @validate(ColonLessGenericValidator())
+    def validator_without_columns(self, **kw):
+        return tg.tmpl_context.form_errors['_the_form']
+
+    @expose('json:')
     @validate(validators={"some_int": validators.Int()})
     def validated_int(self, some_int):
         assert isinstance(some_int, int)
         return dict(response=some_int)
 
-    @expose('json')
+    @expose('json:')
     @validate(validators={"a": validators.Int()})
     def validated_and_unvalidated(self, a, b):
         assert isinstance(a, int)
-        assert isinstance(b, unicode)
+        assert isinstance(b, unicode_text)
         return dict(int=a, str=b)
 
     @expose()
@@ -81,15 +143,15 @@ class BasicTGController(TGController):
     def validate_controller_based_validator(self, *args, **kw):
         return 'ok'
 
-    @expose('json')
-    @validate(validators={"a": validators.Int(), "someemail": validators.Email})
+    @expose('json:')
+    @validate(validators={"a": validators.Int(), "someemail": validators.Email()})
     def two_validators(self, a=None, someemail=None, *args):
-        errors = pylons.tmpl_context.form_errors
-        values =  pylons.tmpl_context.form_values
+        errors = tg.tmpl_context.form_errors
+        values =  tg.tmpl_context.form_values
         return dict(a=a, someemail=someemail,
                 errors=str(errors), values=str(values))
 
-    @expose('json')
+    @expose('json:')
     @validate(validators={"a": validators.Int()})
     def with_default_shadow(self, a, b=None ):
         """A default value should not cause the validated value to disappear"""
@@ -98,13 +160,13 @@ class BasicTGController(TGController):
             'int': a,
         }
 
-    @expose('json')
+    @expose('json:')
     @validate(validators={"e": ColonValidator()})
     def error_with_colon(self, e):
-        errors = pylons.tmpl_context.form_errors
+        errors = tg.tmpl_context.form_errors
         return dict(errors=str(errors))
 
-    @expose('json')
+    @expose('json:')
     @validate(validators={
         "a": validators.Int(),"b":validators.Int(),"c":validators.Int(),"d":validators.Int()
     })
@@ -122,23 +184,23 @@ class BasicTGController(TGController):
     def display_form(self, **kwargs):
         return str(myform.render(values=kwargs))
 
-    @expose('json')
+    @expose('json:')
     @validate(form=myform)
     def process_form(self, **kwargs):
-        kwargs['errors'] = pylons.tmpl_context.form_errors
+        kwargs['errors'] = tg.tmpl_context.form_errors
         return dict(kwargs)
 
-    @expose('json')
+    @expose('json:')
     @validate(form=myform, error_handler=process_form)
     def send_to_error_handler(self, **kwargs):
-        kwargs['errors'] = pylons.tmpl_context.form_errors
+        kwargs['errors'] = tg.tmpl_context.form_errors
         return dict(kwargs)
 
     @expose()
     def tw2form_error_handler(self, **kwargs):
-        return dumps(dict(errors=pylons.tmpl_context.form_errors))
+        return dumps(dict(errors=tg.tmpl_context.form_errors))
 
-    @expose('json')
+    @expose('json:')
     @validate(form=movie_form, error_handler=tw2form_error_handler)
     def send_tw2_to_error_handler(self, **kwargs):
         return 'passed validation'
@@ -146,23 +208,24 @@ class BasicTGController(TGController):
     @expose()
     @validate({'param':tw2c.IntValidator()})
     def tw2_dict_validation(self, **kwargs):
-        return str(pylons.tmpl_context.form_errors)
+        return str(tg.tmpl_context.form_errors)
 
     @expose()
     def set_lang(self, lang=None):
-        pylons.session['tg_lang'] = lang
-        pylons.session.save()
+        tg.session['tg_lang'] = lang
+        tg.session.save()
         return 'ok'
 
-    @expose()
-    @validate(validators=Pwd())
-    def password(self, pwd1, pwd2):
-        if pylons.tmpl_context.form_errors:
-            return "There was an error"
-        else:
-            return "Password ok!"
+    if not PY3:
+        @expose()
+        @validate(validators=Pwd())
+        def password(self, pwd1, pwd2):
+            if tg.tmpl_context.form_errors:
+                return "There was an error"
+            else:
+                return "Password ok!"
 
-    @expose('json')
+    @expose('json:')
     @before_render(lambda rem,params,output:output.update({'GOT_ERROR':'HOOKED'}))
     def hooked_error_handler(self, *args, **kw):
         return dict(GOT_ERROR='MISSED HOOK')
@@ -172,13 +235,23 @@ class BasicTGController(TGController):
     def with_hooked_error_handler(self, *args, **kw):
         return dict(GOT_ERROR='NO ERROR')
 
+    @expose()
+    def error_handler(self, *args, **kw):
+        return 'ERROR HANDLER!'
+
+    @expose('json:')
+    @validate(validators={"some_int": validators.Int()},
+              error_handler=error_handler)
+    def validate_other_error_handler(self, some_int):
+        return dict(response=some_int)
+
 class TestTGController(TestWSGIController):
 
     def setUp(self):
         TestWSGIController.setUp(self)
-        pylons.config.update({
-            'pylons.paths': {'root': data_dir},
-            'pylons.package': 'tests'})
+        tg.config.update({
+            'paths': {'root': data_dir},
+            'package': tests})
         self.app = make_app(BasicTGController)
 
     def test_basic_validation_and_jsonification(self):
@@ -187,6 +260,16 @@ class TestTGController(TestWSGIController):
         resp = self.app.post('/validated_int', form_values)
         assert '{"response": 22}'in resp, resp
 
+    def test_validation_other_error_handler(self):
+        form_values = {"some_int": 'TEXT'}
+        resp = self.app.post('/validate_other_error_handler', form_values)
+        assert 'ERROR HANDLER!'in resp, resp
+
+    def test_validator_without_columns(self):
+        form_values = {"some_int": 22}
+        resp = self.app.post('/validator_without_columns', form_values)
+        assert 'Unknown Error' in resp, resp
+
     def test_for_other_params_after_validation(self):
         """Ensure that both validated and unvalidated data make it through"""
         form_values = {'a': 1, 'b': "string"}
@@ -213,14 +296,14 @@ class TestTGController(TestWSGIController):
         """Ensure that multiple validators are applied correctly"""
         form_values = {'a': '1', 'someemail': "guido@google.com"}
         resp = self.app.post('/two_validators', form_values)
-        content = loads(resp.body)
+        content = loads(resp.body.decode('utf-8'))
         assert content['a'] == 1
 
     def test_validation_errors(self):
         """Ensure that dict validation produces a full set of errors"""
         form_values = {'a': '1', 'someemail': "guido~google.com"}
         resp = self.app.post('/two_validators', form_values)
-        content = loads(resp.body)
+        content = loads(resp.body.decode('utf-8'))
         errors = content.get('errors', None)
         assert errors, 'There should have been at least one error'
         assert 'someemail' in errors, \
@@ -228,17 +311,21 @@ class TestTGController(TestWSGIController):
 
     def test_form_validation(self):
         """Check @validate's handing of ToscaWidget forms instances"""
+        if PY3: raise SkipTest()
+
         form_values = {'title': 'Razer', 'year': "2007"}
         resp = self.app.post('/process_form', form_values)
-        values = loads(resp.body)
+        values = loads(resp.body.decode('utf-8'))
         assert values['year'] == 2007
 
     def test_error_with_colon(self):
         resp = self.app.post('/error_with_colon', {'e':"fakeparam"})
-        assert 'Description' in resp.body, resp.body       
+        assert 'Description' in str(resp.body), resp.body
 
     def test_form_render(self):
         """Test that myform renders properly"""
+        if PY3: raise SkipTest()
+
         resp = self.app.post('/display_form')
         assert 'id="my_form_title.label"' in resp, resp
         assert 'class="fieldlabel required"' in resp, resp
@@ -246,58 +333,70 @@ class TestTGController(TestWSGIController):
 
     def test_form_validation_error(self):
         """Test form validation with error message"""
+        if PY3: raise SkipTest()
+
         form_values = {'title': 'Razer', 'year': "t007"}
         resp = self.app.post('/process_form', form_values)
-        values = loads(resp.body)
+        values = loads(resp.body.decode('utf-8'))
         assert "Please enter an integer value" in values['errors']['year'], \
             'Error message not found: %r' % values['errors']
 
     def test_form_validation_redirect(self):
         """Test form validation error message with redirect"""
+        if PY3: raise SkipTest()
+
         form_values = {'title': 'Razer', 'year': "t007"}
         resp = self.app.post('/send_to_error_handler', form_values)
-        values = loads(resp.body)
+        values = loads(resp.body.decode('utf-8'))
         assert "Please enter an integer value" in values['errors']['year'], \
             'Error message not found: %r' % values['errors']
 
     def test_tw2form_validation(self):
+        if PY3: raise SkipTest()
+
         form_values = {'title': 'Razer', 'year': "t007"}
         resp = self.app.post('/send_tw2_to_error_handler', form_values)
-        values = loads(resp.body)
+        values = loads(resp.body.decode('utf-8'))
         assert "Must be an integer" in values['errors']['year'],\
         'Error message not found: %r' % values['errors']
 
     def test_tw2dict_validation(self):
+        if PY3: raise SkipTest()
+
         resp = self.app.post('/tw2_dict_validation', {'param': "7"})
-        assert '{}' in resp.body
+        assert '{}' in str(resp.body)
 
         resp = self.app.post('/tw2_dict_validation', {'param': "hello"})
-        assert 'Must be an integer' in resp.body
+        assert 'Must be an integer' in str(resp.body)
 
     def test_form_validation_translation(self):
+        if PY3: raise SkipTest()
+
         """Test translation of form validation error messages"""
         form_values = {'title': 'Razer', 'year': "t007"}
         # check with language set in request header
         resp = self.app.post('/process_form', form_values,
             headers={'Accept-Language': 'de,ru,it'})
-        values = loads(resp.body)
-        assert u"Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
+        values = loads(resp.body.decode('utf-8'))
+        assert "Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
             'No German error message: %r' % values['errors']
         resp = self.app.post('/process_form', form_values,
             headers={'Accept-Language': 'ru,de,it'})
-        values = loads(resp.body)
-        assert u"Введите числовое значение" in values['errors']['year'], \
+        values = loads(resp.body.decode('utf-8'))
+        assert u_("Введите числовое значение") in values['errors']['year'], \
             'No Russian error message: %r' % values['errors']
         # check with language set in session
         self.app.post('/set_lang/de')
         resp = self.app.post('/process_form', form_values,
             headers={'Accept-Language': 'ru,it'})
-        values = loads(resp.body)
-        assert u"Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
+        values = loads(resp.body.decode('utf-8'))
+        assert "Bitte eine ganze Zahl eingeben" in values['errors']['year'], \
             'No German error message: %r' % values['errors']
 
     def test_form_validation_error(self):
         """Test schema validation"""
+        if PY3: raise SkipTest()
+
         form_values = {'pwd1': 'me', 'pwd2': 'you'}
         resp = self.app.post('/password', form_values)
         assert "There was an error" in resp, resp
@@ -312,4 +411,8 @@ class TestTGController(TestWSGIController):
 
     def test_hook_after_validation_error(self):
         resp = self.app.post('/with_hooked_error_handler?v=a')
-        assert 'HOOKED' in resp, resp
\ No newline at end of file
+        assert 'HOOKED' in resp, resp
+
+    def test_validation_error_has_message(self):
+        e = TGValidationError('This is a validation error')
+        assert str(e) == 'This is a validation error'
\ No newline at end of file
