@@ -4,17 +4,15 @@
 URL definition and browser redirection are defined here.
 
 """
+import re
+from webob.exc import status_map
 
-import urllib
-from warnings import warn
-
-
-import pylons
+import tg
 
+from tg._compat import string_type, url_encode, unicode_text, bytes_
 from tg.exceptions import HTTPFound
 
-
-def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
+def _smart_str(s):
     """
     Returns a bytestring version of 's', encoded as specified in 'encoding'.
 
@@ -23,49 +21,51 @@ def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
     This function was borrowed from Django.
 
     """
-    if strings_only and (s is None or isinstance(s, int)):
-        return s
-    elif not isinstance(s, basestring):
+    if not isinstance(s, string_type):
         try:
-            return str(s)
+            return bytes_(s)
         except UnicodeEncodeError:
             if isinstance(s, Exception):
                 # An Exception subclass containing non-ASCII data that doesn't
                 # know how to print itself properly. We shouldn't raise a
                 # further exception.
-                return ' '.join([smart_str(arg, encoding, strings_only,
-                        errors) for arg in s])
-            return unicode(s).encode(encoding, errors)
-    elif isinstance(s, unicode):
-        r = s.encode(encoding, errors)
-        return r
-    elif s and encoding != 'utf-8':
-        return s.decode('utf-8', errors).encode(encoding, errors)
+                return ' '.join([_smart_str(arg).decode('utf-8') for arg in s.args]).encode('utf-8', 'strict')
+            return unicode_text(s).encode('utf-8', 'strict')
+    elif isinstance(s, unicode_text):
+        return s.encode('utf-8', 'strict')
     else:
         return s
 
 
-def generate_smart_str(params):
-    for key, value in params.iteritems():
+def _generate_smart_str(params):
+    for key, value in params.items():
         if value is None:
             continue
         if isinstance(value, (list, tuple)):
             for item in value:
-                yield smart_str(key), smart_str(item)
+                yield _smart_str(key), _smart_str(item)
         else:
-            yield smart_str(key), smart_str(value)
+            yield _smart_str(key), _smart_str(value)
 
 
-def urlencode(params):
+def _urlencode(params):
     """
     A version of Python's urllib.urlencode() function that can operate on
     unicode strings. The parameters are first case to UTF-8 encoded strings and
     then encoded as per normal.
     """
-    return urllib.urlencode([i for i in generate_smart_str(params)])
+    return url_encode([i for i in _generate_smart_str(params)])
 
+def _build_url(environ, base_url='/', params=None):
+    if base_url.startswith('/'):
+        base_url = environ['SCRIPT_NAME'] + base_url
 
-def url(base_url='/', params={}, qualified=False):
+    if params:
+        return '?'.join((base_url, _urlencode(params)))
+
+    return base_url
+
+def url(base_url='/', params=None, qualified=False):
     """Generate an absolute URL that's specific to this application.
 
     The URL function takes a string (base_url) and, appends the
@@ -73,16 +73,13 @@ def url(base_url='/', params={}, qualified=False):
     parameters passed into the params dict.
 
     """
-    if not isinstance(base_url, basestring) and hasattr(base_url, '__iter__'):
+    if not isinstance(base_url, string_type) and hasattr(base_url, '__iter__'):
         base_url = '/'.join(base_url)
 
-    if base_url.startswith('/'):
-        base_url = pylons.request.environ['SCRIPT_NAME'] + base_url
-        if qualified:
-            base_url = pylons.request.host_url + base_url
-
-    if params:
-        return '?'.join((base_url, urlencode(params)))
+    req = tg.request._current_obj()
+    base_url = _build_url(req.environ, base_url, params)
+    if qualified:
+        base_url = req.host_url + base_url
 
     return base_url
 
@@ -148,7 +145,7 @@ def lurl(base_url=None, params=None):
     return LazyUrl(base_url, params)
 
 
-def redirect(base_url='/', params={}, **kwargs):
+def redirect(base_url='/', params={}, redirect_with=HTTPFound, **kwargs):
     """Generate an HTTP redirect.
 
     The function raises an exception internally,
@@ -165,27 +162,58 @@ def redirect(base_url='/', params={}, **kwargs):
         params.update(kwargs)
 
     new_url = url(base_url, params=params)
-    found = HTTPFound(location=new_url).exception
-    raise found
+    raise redirect_with(location=new_url)
 
+IF_NONE_MATCH = re.compile('(?:W/)?(?:"([^"]*)",?\s*)')
+def etag_cache(key=None):
+    """Use the HTTP Entity Tag cache for Browser side caching
+
+    If a "If-None-Match" header is found, and equivilant to ``key``,
+    then a ``304`` HTTP message will be returned with the ETag to tell
+    the browser that it should use its current cache of the page.
+
+    Otherwise, the ETag header will be added to the response headers.
+    """
+    if_none_matches = IF_NONE_MATCH.findall(tg.request.environ.get('HTTP_IF_NONE_MATCH', ''))
+    response = tg.response._current_obj()
+    response.headers['ETag'] = '"%s"' % key
+    if str(key) in if_none_matches:
+        response.headers.pop('Content-Type', None)
+        response.headers.pop('Cache-Control', None)
+        response.headers.pop('Pragma', None)
+        raise status_map[304]()
+
+def abort(status_code=None, detail="", headers=None, comment=None):
+    """Aborts the request immediately by returning an HTTP exception
+
+    In the event that the status_code is a 300 series error, the detail
+    attribute will be used as the Location header should one not be
+    specified in the headers attribute.
+
+    """
+    exc = status_map[status_code](detail=detail, headers=headers,
+                                  comment=comment)
+    raise exc
 
 def use_wsgi_app(wsgi_app):
-    return wsgi_app(pylons.request.environ, pylons.request.start_response)
+    return tg.request.get_response(wsgi_app)
 
 
+NullTranslations = tg_gettext = None
+
 # Idea stolen from Pylons
 def pylons_formencode_gettext(value):
-    from pylons.i18n import ugettext as pylons_gettext
-    from gettext import NullTranslations
-
-    trans = pylons_gettext(value)
+    global NullTranslations, tg_gettext
+    if tg_gettext is None:
+        from tg.i18n import ugettext as tg_gettext
+        from gettext import NullTranslations
 
+    trans = tg_gettext(value)
     # Translation failed, try formencode
     if trans == value:
-
         try:
-            fetrans = pylons.tmpl_context.formencode_translation
-        except AttributeError:
+            fetrans = tg.tmpl_context.formencode_translation
+        except (AttributeError, TypeError):
             # the translator was not set in the Pylons context
             # we are certainly in the test framework
             # let's make sure won't return something that is ok with the caller
@@ -194,8 +222,9 @@ def pylons_formencode_gettext(value):
         if not fetrans:
             fetrans = NullTranslations()
 
-        trans = fetrans.ugettext(value)
+        translator_gettext = getattr(fetrans, 'ugettext', fetrans.gettext)
+        trans = translator_gettext(value)
 
     return trans
 
-__all__ = ['url', 'lurl', 'redirect', 'etag_cache']
+__all__ = ['url', 'lurl', 'redirect', 'etag_cache', 'abort']
