@@ -8,25 +8,20 @@ the functions they wrap, and then the DecoratedController provides the hooks
 needed to support these decorators.
 
 """
-from warnings import warn
-
 from decorator import decorator
-from formencode import variabledecode
-from paste.util.mimeparse import best_match
-from pylons.configuration import config
-from pylons import request, response
-from pylons.controllers.util import abort
-from webob.exc import HTTPMethodNotAllowed
 
-from tg import tmpl_context
-from tg.paginate import Page
-from tg.util import Bunch, partial
+from webob.exc import HTTPUnauthorized, HTTPMethodNotAllowed, HTTPMovedPermanently
+from tg.support.paginate import Page
+from tg.configuration import config
+from tg.controllers.util import abort, redirect
+from tg import tmpl_context, request, response
+from tg.util import partial, Bunch
 from tg.configuration.sqla.balanced_session import force_request_engine
 from tg.flash import flash
-
+from tg.caching import beaker_cache, cached_property
 from tg.predicates import NotAuthorizedError
-
-from pylons.decorators.cache import beaker_cache
+from tg._compat import im_func, unicode_text
+from webob.acceptparse import Accept
 
 import logging
 log = logging.getLogger(__name__)
@@ -38,6 +33,7 @@ class Decoration(object):
         self.controller = controller
         self.engines = {}
         self.engines_keys = []
+        self.default_engine = None
         self.custom_engines = {}
         self.render_custom_format = None
         self.validation = None
@@ -48,9 +44,11 @@ class Decoration(object):
                           after_render=[])
 
     def get_decoration(cls, func):
-        if not hasattr(func, 'decoration'):
-            func.decoration = cls(func)
-        return func.decoration
+        try:
+            dec = func.decoration
+        except:
+            dec = func.decoration = cls(func)
+        return dec
     get_decoration = classmethod(get_decoration)
 
     @property
@@ -58,9 +56,9 @@ class Decoration(object):
         return bool(self.engines) or bool(self.custom_engines)
 
     def merge(self, deco):
-        self.engines = dict(deco.engines.items() + self.engines.items())
+        self.engines = dict(tuple(deco.engines.items()) + tuple(self.engines.items()))
         self.engines_keys = sorted(self.engines, reverse=True)
-        self.custom_engines = dict(deco.custom_engines.items() + self.engines.items())
+        self.custom_engines = dict(tuple(deco.custom_engines.items()) + tuple(self.engines.items()))
 
         #inherit all the parent hooks
         #parent hooks before current hooks so that they get called before
@@ -70,18 +68,19 @@ class Decoration(object):
         if not self.validation:
             self.validation = deco.validation
 
-    def run_hooks(self, hook, *l, **kw):
-        for func in config.get('hooks', {}).get(hook, []):
-            func(*l, **kw)
+    def run_hooks(self, tgl, hook, *l, **kw):
+        #run system wide hooks
+        try:
+            syswide_hooks = tgl.config['hooks'][hook]
+            for func in syswide_hooks:
+                func(*l, **kw)
+        except KeyError:
+            pass
+
+        #run controller hooks
         for func in self.hooks[hook]:
             func(*l, **kw)
 
-    def wrap_controller(self, controller):
-        controller_callable = controller
-        for wrapper in config.get('controller_wrappers', []):
-            controller_callable = wrapper(self, controller_callable)
-        return controller_callable
-
     def register_template_engine(self,
             content_type, engine, template, exclude_names, render_params):
         """Registers an engine on the controller.
@@ -120,6 +119,12 @@ class Decoration(object):
 
         self.engines[content_type] = (engine, template, exclude_names, render_params or {})
 
+        #Avoid engine lookup if we have only one engine registered
+        if len(self.engines) == 1:
+            self.default_engine = content_type
+        else:
+            self.default_engine = None
+
         # this is a work-around to make text/html prominent in respect
         # to other common choices when they have the same weight for
         # paste.util.mimeparse.best_match.
@@ -152,7 +157,7 @@ class Decoration(object):
         self.custom_engines[custom_format or '"*/*"'] = (
             content_type, engine, template, exclude_names, render_params or {})
 
-    def lookup_template_engine(self, request):
+    def lookup_template_engine(self, tgl):
         """Return the template engine data.
 
         Provides a convenience method to get the proper engine,
@@ -160,16 +165,11 @@ class Decoration(object):
         tg_format (which is pulled off of the request headers).
 
         """
-
-        response_type = getattr(request, 'response_type', None)
-        if response_type is not None and response_type in self.engines:
-            accept_types = request.response_type
-        else:
-            accept_types = request.headers.get('accept', '*/*')
+        request = tgl.request
+        response = tgl.response
 
         try:
-            render_custom_format = request._render_custom_format[
-                self.controller]
+            render_custom_format = request._render_custom_format[self.controller]
         except:
             render_custom_format = self.render_custom_format
 
@@ -177,27 +177,29 @@ class Decoration(object):
             (content_type, engine, template, exclude_names, render_params
                 ) = self.custom_engines[render_custom_format]
         else:
-            if self.engines:
-                content_type = best_match(self.engines_keys, accept_types)
+            if self.default_engine:
+                content_type = self.default_engine
+            elif self.engines:
+                if request._response_type and request._response_type in self.engines:
+                    accept_types = request._response_type
+                else:
+                    accept_types = request.headers.get('accept', '*/*')
+                content_type = Accept(accept_types).best_match(self.engines_keys, self.engines_keys[0])
             else:
                 content_type = 'text/html'
 
-            if content_type == 'CUSTOM/LEAVE':
-                warn('@expose(CUSTOM_CONTENT_TYPE) is no longer needed'
-                     ' and should be replaced with @expose()')
-
             # check for overridden content type from the controller call
-            controller_content_type = response.headers.get('Content-Type')
-
-            if controller_content_type:
-                # make sure we handle types like 'text/html; charset=utf-8'
-                content_type = controller_content_type.split(';', 1)[0]
+            try:
+                controller_content_type = response.headers['Content-Type']
+                # make sure we handle content_types like 'text/html; charset=utf-8'
+                content_type = controller_content_type.split(';')[0]
+            except KeyError:
+                pass
 
             # check for overridden templates
             try:
-                (engine, template, exclude_names, render_params
-                    ) = request._override_mapping[
-                            self.controller][content_type.split(';', 1)[0]]
+                cnt_override_mapping = request._override_mapping[self.controller]
+                engine, template, exclude_names, render_params = cnt_override_mapping[content_type.split(";")[0]]
             except (AttributeError, KeyError):
                 (engine, template, exclude_names, render_params
                     ) = self.engines.get(content_type, (None,) * 4)
@@ -205,7 +207,7 @@ class Decoration(object):
         if 'charset' not in content_type and (content_type.startswith('text')
                 or  content_type in ('application/xhtml+xml',
                         'application/xml', 'application/json')):
-            content_type = '%s; charset=utf-8' % content_type
+            content_type += '; charset=utf-8'
 
         return content_type, engine, template, exclude_names, render_params
 
@@ -413,7 +415,7 @@ def use_custom_format(controller, custom_format):
         render_custom_format = request._render_custom_format
     except AttributeError:
         render_custom_format = request._render_custom_format = {}
-    render_custom_format[controller.im_func] = custom_format
+    render_custom_format[im_func(controller)] = custom_format
 
 
 def override_template(controller, template):
@@ -438,14 +440,14 @@ def override_template(controller, template):
     except:
         return
 
-    for content_type, content_engine in engines.iteritems():
+    for content_type, content_engine in engines.items():
         tmpl = template.split(':', 1)
         tmpl.extend(content_engine[2:])
         try:
             override_mapping = request._override_mapping
         except AttributeError:
             override_mapping = request._override_mapping = {}
-        override_mapping.setdefault(controller.im_func, {}).update({content_type: tmpl})
+        override_mapping.setdefault(im_func(controller), {}).update({content_type: tmpl})
 
 
 class validate(object):
@@ -584,8 +586,7 @@ class paginate(object):
 
         paginator = request.paginators[self.name]
         collection = output[self.name]
-        page = Page(collection, paginator.paginate_page,
-            paginator.paginate_items_per_page, controller='/')
+        page = Page(collection, paginator.paginate_page, paginator.paginate_items_per_page)
         page.kwargs = paginator.paginate_params
         if self.page_param != 'name':
             page.pager = partial(page.pager, page_param=self.page_param)
@@ -593,34 +594,16 @@ class paginate(object):
             tmpl_context.paginators = Bunch()
         tmpl_context.paginators[self.name] = output[self.name] = page
 
-
-@decorator
-def postpone_commits(func, *args, **kwargs):
-    """Turns SQLAlchemy commits into flushes in the decorated method.
-
-    This has the end-result of postponing the commit to the normal TG2
-    transaction boundary. """
-
-    #TODO: Test and document this.
-    s = config.get('DBSession', None)
-    assert hasattr(s, 'commit')
-    old_commit = s.commit
-    s.commit = s.flush
-    retval = func(*args, **kwargs)
-    s.commit = old_commit
-    return retval
-
-
 @before_validate
 def https(remainder, params):
     """Ensure that the decorated method is always called with https."""
-    from tg.controllers import redirect
     if request.scheme.lower() == 'https': return
     if request.method.upper() == 'GET':
         redirect('https' + request.url[len(request.scheme):])
-    raise HTTPMethodNotAllowed(headers=dict(Allow='GET')).exception
+    raise HTTPMethodNotAllowed(headers=dict(Allow='GET'))
 
 
+_variabledecode = None
 @before_validate
 def variable_decode(remainder, params):
     """Best-effort formencode.variabledecode on the params before validation.
@@ -632,8 +615,12 @@ def variable_decode(remainder, params):
     to {'foo-1':'1', 'foo-2':'2', 'foo':['1', '2']}.
 
     """
+    global _variabledecode
+    if _variabledecode is None:
+        from formencode import variabledecode as _variabledecode
+
     try:
-        new_params = variabledecode.variable_decode(params)
+        new_params = _variabledecode.variable_decode(params)
         params.update(new_params)
     except:
         pass
@@ -660,9 +647,9 @@ def without_trailing_slash(remainder, params):
     In addition, the URL http://localhost:8080/sample/1/ redirects to http://localhost:8080/sample/1
 
     """
-    if request.method == 'GET' and request.path.endswith('/') and not(request.response_type) and len(request.params)==0:
-        from tg.controllers import redirect
-        redirect(request.url[:-1])
+    req = request._current_obj()
+    if req.method == 'GET' and req.path.endswith('/') and not(req._response_type) and len(req.params)==0:
+        redirect(request.url[:-1], redirect_with=HTTPMovedPermanently)
 
 
 @before_validate
@@ -686,17 +673,12 @@ def with_trailing_slash(remainder, params):
     In addition, the URL http://localhost:8080/sample/1 redirects to http://localhost:8080/sample/1/
 
     """
-    if (request.method == 'GET'
-        and not(request.path.endswith('/'))
-        and not(request.response_type)
-        and len(request.params)==0):
-        from tg.controllers import redirect
-        redirect(request.url+'/')
+    req = request._current_obj()
+    if (req.method == 'GET' and not(req.path.endswith('/')) and not(req._response_type) and len(req.params)==0):
+        redirect(request.url+'/', redirect_with=HTTPMovedPermanently)
 
 
 #{ Authorization decorators
-
-
 class _BaseProtectionDecorator(object):
     default_denial_handler = None
 
@@ -712,14 +694,13 @@ class _BaseProtectionDecorator(object):
         which represents a message on why authorization was denied.
 
         """
-
         self.predicate = predicate
         self.denial_handler = denial_handler or self.default_denial_handler
         self.smart_denial = smart_denial
 
-
 class require(_BaseProtectionDecorator):
-    """TurboGears-specific action protector.
+    """
+    TurboGears-specific action protector.
 
     The default authorization denial handler of this protector will flash
     the message of the unmet predicate with ``warning`` or ``error`` as the
@@ -728,7 +709,6 @@ class require(_BaseProtectionDecorator):
     See :class:`allow_only` for controller-wide authorization.
 
     """
-
     def __call__(self, action_):
         return decorator(self.wrap_action, action_)
 
@@ -737,18 +717,16 @@ class require(_BaseProtectionDecorator):
 
         try:
             self.predicate.check_authorization(req.environ)
-        except NotAuthorizedError, e:
-            reason = unicode(e)
+        except NotAuthorizedError as e:
+            reason = unicode_text(e)
             if req.environ.get('repoze.who.identity'):
                 # The user is authenticated.
                 code = 403
             else:
                 # The user is not authenticated.
                 code = 401
-            if self.denial_handler:
-                response.status = code
-                return self.denial_handler(reason)
-            abort(code, comment=reason)
+            response.status = code
+            return self.denial_handler(reason)
         return action_(*args, **kwargs)
 
     def default_denial_handler(self, reason):
@@ -761,51 +739,6 @@ class require(_BaseProtectionDecorator):
                 flash(reason, status=status)
         abort(response.status_int, reason)
 
-
-class allow_only(_BaseProtectionDecorator):
-    """TurboGears controller wide protector.
-
-    The default authorization denial handler of this protector will flash
-    the message of the unmet predicate with ``warning`` or ``error`` as the
-    flash status if the HTTP status code is 401 or 403, respectively, since
-    by default the ``__before__`` method of the controller is decorated with
-    :class:`require`.
-
-    If the controller class has the ``_failed_authorization`` *class method*,
-    it will replace the default denial handler.
-
-    """
-    protector = require
-
-    def __call__(self, cls, *args, **kwargs):
-        if hasattr(self.protector, 'predicate'):
-            cls.allow_only = self.protector.predicate
-        if hasattr(cls, '_failed_authorization'):
-            self.denial_handler = cls._failed_authorization
-        sup = super(allow_only, self)
-        if hasattr(sup, '__call__'):
-            return super(allow_only, self).__call__(cls, *args, **kwargs)
-
-
-class cached_property(object):
-    """Cached property executing the getter only once."""
-
-    def __init__(self, func):
-        self.__name__ = func.__name__
-        self.__module__ = func.__module__
-        self.__doc__ = func.__doc__
-        self.func = func
-
-    def __get__(self, obj, type=None):
-        if obj is None:
-            return self
-        try:
-            value = obj.__dict__[self.__name__]
-        except KeyError:
-            value = obj.__dict__[self.__name__] = self.func(obj)
-        return value
-
-
 class with_engine(object):
     """
     Decorator to force usage of a specific database engine
@@ -846,5 +779,4 @@ class with_engine(object):
         decoration = Decoration.get_decoration(func)
         decoration.register_hook('before_validate', self.before_validate)
         return func
-
 #}
