@@ -1,6 +1,6 @@
 """TurboGears project related information"""
-version = "2.2.2"
-description = "Next generation TurboGears built on Pylons"
+version = "2.3.0"
+description = "Next generation TurboGears"
 long_description="""
 TurboGears brings together a best of breed python tools
 to create a flexible, full featured, and easy to use web
@@ -18,14 +18,14 @@ develeopment, back database development and everything in between:
  * powerful Object Relational Mappers (SQLAlchemy)
 
 The latest development version is available in the
-`TurboGears Git repository`_.
+`TurboGears Git repositories`_.
 
-.. _TurboGears Git repository:
-    https://sourceforge.net/p/turbogears2/tg2/
+.. _TurboGears Git repositories:
+    https://github.com/TurboGears
 """
 url="http://www.turbogears.org/"
 author= "Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, Michael Pedersen, Alessandro Molina, and the TurboGears community"
 email = "mark.ramm@gmail.com, alberto@toscat.net, m.pedersen@icelus.org, amol@turbogears.org"
-copyright = """Copyright 2005-2011 Kevin Dangoor,
+copyright = """Copyright 2005-2013 Kevin Dangoor,
 Alberto Valverde, Mark Ramm, Christopher Perkins and contributors"""
 license = "MIT"
