@@ -1,15 +1,15 @@
-from urllib import quote_plus
+try:
+    from urllib import quote_plus
+except ImportError: #pragma: no cover
+    from urllib.parse import quote_plus
 
 from paste.deploy.converters import asbool
-from webhelpers.html import literal
+from markupsafe import Markup
 
 import tg
 from tg import predicates
 from tg.util import Bunch
 
-from webhelpers.html import literal
-
-
 class MissingRendererError(Exception):
     def __init__(self, template_engine):
         Exception.__init__(self,
@@ -19,23 +19,6 @@ class MissingRendererError(Exception):
             template_engine=template_engine))
         self.template_engine = template_engine
 
-
-class DeprecatedFlashVariable(object):
-    def __init__(self, callable, msg):
-        self.callable = callable
-        self.msg = msg
-
-    def __unicode__(self):
-        import warnings
-        warnings.warn(self.msg, DeprecationWarning, 2)
-        return unicode(self.callable())
-
-    def __nonzero__(self):
-        import warnings
-        warnings.warn(self.msg, DeprecationWarning, 2)
-        return bool(self.callable())
-
-
 def _get_tg_vars():
     """Create a Bunch of variables that should be available in all templates.
 
@@ -71,43 +54,35 @@ def _get_tg_vars():
 
     """
 
-    req = tg.request._current_obj()
-    conf = tg.config._current_obj()
-    tmpl_context = tg.tmpl_context._current_obj()
-    app_globals = tg.app_globals._current_obj()
-    translator = tg.translator._current_obj()
-    response = tg.response._current_obj()
+    tgl = tg.request_local.context._current_obj()
+    req = tgl.request
+    conf = tgl.config
+    tmpl_context = tgl.tmpl_context
+    app_globals = tgl.app_globals
+    translator = tgl.translator
+    response = tgl.response
+    session = tgl.session
 
     try:
         h = conf['package'].lib.helpers
-    except AttributeError, ImportError:
+    except (AttributeError, ImportError):
         h = Bunch()
 
     # TODO: Implement user_agent and other missing features.
     tg_vars = Bunch(
         config=tg.config,
         flash_obj=tg.flash,
-        flash=DeprecatedFlashVariable(
-            lambda: tg.flash.message,
-            "flash is deprecated, please use flash_obj.message instead "
-            "or use the new flash_obj.render() method"
-            ),
-        flash_status=DeprecatedFlashVariable(
-            lambda: 'status_' + tg.flash.status,
-            "flash_status is deprecated, please use flash_obj.status instead "
-            "or use the new flash_obj.render() method"
-            ),
         quote_plus=quote_plus,
         url=tg.url,
         # this will be None if no identity
-        identity=req.environ.get('repoze.who.identity'),
-        session=tg.session,
-        locale=req.accept_language.best_matches(),
-        errors=getattr(tmpl_context, "form_errors", {}),
-        inputs=getattr(tmpl_context, "form_values", {}),
-        request=req,
-        auth_stack_enabled='repoze.who.plugins' in req.environ,
-        predicates=predicates)
+        identity = req.environ.get('repoze.who.identity'),
+        session = session,
+        locale = req.plain_languages,
+        errors = getattr(tmpl_context, "form_errors", {}),
+        inputs = getattr(tmpl_context, "form_values", {}),
+        request = req,
+        auth_stack_enabled = 'repoze.who.plugins' in req.environ,
+        predicates = predicates)
 
     root_vars = Bunch(
         c=tmpl_context,
@@ -117,6 +92,7 @@ def _get_tg_vars():
         config=conf,
         app_globals=app_globals,
         g=app_globals,
+        session=session,
         url=tg.url,
         helpers=h,
         h=h,
@@ -126,11 +102,6 @@ def _get_tg_vars():
         _=tg.i18n.ugettext,
         N_=tg.i18n.gettext_noop)
 
-    econf = conf['pylons.environ_config']
-    if 'beaker.session' in req.environ or \
-        ('session' in econf and econf['session'] in req.environ):
-        root_vars['session'] = tg.session._current_obj()
-
     # Allow users to provide a callable that defines extra vars to be
     # added to the template namespace
     variable_provider = conf.get('variable_provider', None)
@@ -138,13 +109,16 @@ def _get_tg_vars():
         root_vars.update(variable_provider())
     return root_vars
 
-# Monkey patch pylons_globals for cases when pylons.templating is used
-# instead of tg.render to programmatically render templates.
-import pylons
-pylons.templating.pylons_globals = _get_tg_vars
+#Monkey patch pylons_globals for cases when pylons.templating is used
+#instead of tg.render to programmatically render templates.
+try: #pragma: no cover
+    import pylons
+    import pylons.templating
+    pylons.templating.pylons_globals = _get_tg_vars
+except ImportError:
+    pass
 # end monkeying around
 
-
 def render(template_vars, template_engine=None, template_name=None, **kwargs):
     config = tg.config._current_obj()
 
@@ -158,6 +132,11 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
             # engine list, warn developer
             raise MissingRendererError(template_engine)
 
+    if not render_function:
+        # getting the default renderer, if no engine was defined in @expose()
+        template_engine = config['default_renderer']
+        render_function = config['render_functions'][template_engine]
+
     if not template_vars:
         template_vars = {}
 
@@ -175,11 +154,6 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
         tg_vars = _get_tg_vars()
         tg_vars.update(template_vars)
 
-    if not render_function:
-        # getting the default renderer, if no engine was defined in @expose()
-        render_function = config[
-            'render_functions'][config['default_renderer']]
-
     kwargs['result'] = render_function(template_name, tg_vars, **kwargs)
 
     for func in config.get('hooks', {}).get('after_render_call', []):
@@ -229,18 +203,21 @@ def cached_template(template_name, render_func, ns_options=(),
 
     """
     # If one of them is not None then the user did set something
-    if (cache_key is not None
-            or cache_expire is not None or cache_type is not None):
-        if not cache_type:
-            cache_type = 'dbm'
+    if cache_key is not None or cache_type is not None or cache_expire is not None:
+        get_cache_kw = {}
+        if cache_type is not None:
+            get_cache_kw['type'] = cache_type
+
         if not cache_key:
             cache_key = 'default'
         if cache_expire == 'never':
             cache_expire = None
+
         namespace = template_name
         for name in ns_options:
             namespace += str(kwargs.get(name))
-        cache = tg.cache.get_cache(namespace, type=cache_type)
+
+        cache = tg.cache.get_cache(namespace, **get_cache_kw)
         content = cache.get_value(cache_key, createfunc=render_func,
             expiretime=cache_expire)
         return content
@@ -248,7 +225,7 @@ def cached_template(template_name, render_func, ns_options=(),
         return render_func()
 
 
-class RenderChameleonGenshi(object):
+class RenderChameleonGenshi(object): #pragma: no cover
     """Singleton that can be called as the Chameleon-Genshi render function."""
 
     format_for_content_type = {
@@ -283,7 +260,7 @@ class RenderChameleonGenshi(object):
 
         def render_template():
             template = self.load_template(template_name, format=format)
-            return literal(template.render(**template_vars))
+            return Markup(template.render(**template_vars))
 
         return cached_template(template_name, render_template,
                                ns_options=('doctype', 'method'), **kwargs)
@@ -369,6 +346,7 @@ class RenderGenshi(object):
 
         """
         response = tg.response._current_obj()
+        config = tg.config._current_obj()
 
         template_vars.update(self.genshi_functions)
 
@@ -397,7 +375,7 @@ class RenderGenshi(object):
 
         def render_template():
             template = self.load_template(template_name)
-            return literal(template.generate(**template_vars).render(
+            return Markup(template.generate(**template_vars).render(
                     doctype=doctype, method=method, encoding=None))
 
         return cached_template(template_name, render_template,
@@ -417,7 +395,7 @@ def render_mako(template_name, globs,
     def render_template():
         # Grab a template reference
         template = globs['app_globals'].mako_lookup.get_template(template_name)
-        return literal(template.render_unicode(**globs))
+        return Markup(template.render_unicode(**globs))
 
     return cached_template(template_name, render_template, cache_key=cache_key,
                            cache_type=cache_type, cache_expire=cache_expire)
@@ -440,7 +418,7 @@ def render_kajiki(template_name, globs, cache_key=None,
     def render_template():
         # Grab a template reference
         template = globs['app_globals'].kajiki_loader.load(template_name)
-        return literal(template(globs).render())
+        return Markup(template(globs).render())
 
     return cached_template(template_name, render_template,
         cache_key=cache_key, cache_type=cache_type, cache_expire=cache_expire,
@@ -459,7 +437,7 @@ def render_jinja(template_name, globs, cache_key=None,
     def render_template():
         # Grab a template reference
         template = globs['app_globals'].jinja2_env.get_template(template_name)
-        return literal(template.render(**globs))
+        return Markup(template.render(**globs))
 
     return cached_template(template_name, render_template,
         cache_key=cache_key, cache_type=cache_type, cache_expire=cache_expire)
